--***********************************************************
--** PseudonymousEd, the Dev
--** Playable Arcade Machines 41.50
--** Context menu for playing an arcade machine
--***********************************************************

PAMPlayGameMenu = {};

-- Add play option to arcade machine context menus
PAMPlayGameMenu.doBuildMenu = function(player, context, worldobjects)

	-- Do not worry about the following line	
	local USE_CUSTOM_ANIMATION = false

	--****************************************
	--* ATTENTION: If you have installed
	--*      the custom animations manually,
	--*      make sure to uncomment
	--*      (remove -- from the front of)
	--*      local USE_CUSTOM_ANIMATION = true
	--****************************************
	
	-- To enable Custom Animation, remove the -- from the following line:
	--local USE_CUSTOM_ANIMATION = true

	
	local thisPlayer = getSpecificPlayer(player);
	local thisObject = worldobjects[1];

	if thisObject:getSprite() then

		local properties = thisObject:getSprite():getProperties()

		if properties ~= nil then
			local groupName = nil
			local customName = nil
			
			if properties:Is("GroupName") then
				groupName = properties:Val("GroupName")
				--print("PseudoEdPAM: Sprite GroupName: " .. groupName);
			end

			if properties:Is("CustomName") then
				customName = properties:Val("CustomName")
				--print("PseudoEdPAM: Sprite CustomName: " .. customName);
			end
			
			-- For Arcade Machines, Custom Name = "Machine" and the Group Name is more specific 
			if customName == "Machine" then

				if not ((SandboxVars.ElecShutModifier > -1 and
						 GameTime:getInstance():getNightsSurvived() < SandboxVars.ElecShutModifier) or
					thisObject:getSquare():haveElectricity()) then
					-- print("PseudoEdPAM: This object is unpowered");
					return
				end
				
				local machine = nil;
				local soundFile = nil;
				local contextMenu = nil;
				local actionType = nil;

				if groupName == "Kaboom Arcade" then
					soundFile = "PAMkaboomplay"
					soundEnd = "PAMkaboomend"
					contextMenu = "ContextMenu_Play_Kaboom"
					actionType = "Loot"
					if USE_CUSTOM_ANIMATION then
						actionType = "PlayArcade"
					end
				elseif groupName == "Dr. Oids Arcade" then
					soundFile = "PAMdroidsplay"
					soundEnd = "PAMdroidsend"
					contextMenu = "ContextMenu_Play_DrOids"
					actionType = "Loot"
					if USE_CUSTOM_ANIMATION then
						actionType = "PlayArcade"
					end
				elseif groupName == "PAWS Pinball" then
					local spriteName = thisObject:getSprite():getName()
					-- TODO Fix and enable for the back of the Pinball machine
					if (spriteName == "recreational_01_25" or spriteName == "recreational_01_26") then
						-- print("PseudoEdPAM: Currently ignoring pinball backs.  Returning")
						-- -- print("PseudoEdPAM:  Not ignoring pinball backs right now")
						-- Currently unable to put player in correct position for back of pinball machine
						return
					end
					
					soundFile = "PAMpinballplay"
					soundEnd = "PAMpinballend"
					contextMenu = "ContextMenu_Play_PAWS_Pinball"
					actionType = "Loot"
					if USE_CUSTOM_ANIMATION then
						actionType = "PlayPinball"
					end
				else
					print("PseudoEdPAM: Found unplayable machine with groupName " .. groupName)
					return
				end -- if groupName

				--if USE_CUSTOM_ANIMATION then
				--	print("PseudoEdPAM: Enabled USE_CUSTOM_ANIMATION")
				--else
				--	print("PseudoEdPAM: Using default animations")
				--end
				
				machine = thisObject
				context:addOption(getText(contextMenu),
								  worldobjects,
								  PAMPlayGameMenu.onPlayGame,
								  getSpecificPlayer(player),
								  machine,
								  soundFile,
								  soundEnd,
								  actionType,
								  960,
								  8,
								  5);
				
			end -- if customName == "Machine"
		end -- if properties ~= nil
	end -- if thisObject:getSprite()
end -- PAMPlayGameMenu.doBuildMenu

-- Moreso than the rest of the code, I don't really know what I'm doing here
PAMPlayGameMenu.walkToFront = function(thisPlayer, thisObject)
	local frontSquare = nil
	local controllerSquare = nil
	local spriteName = thisObject:getSprite():getName()
	if not spriteName then
		return false
	end

	local properties = thisObject:getSprite():getProperties()
	
	local facing = nil
	if properties:Is("Facing") then
		facing = properties:Val("Facing")
		--print("PseudoEdPAM: walkToFront Sprite Facing: " .. facing)
	else
		print("PseudoEdPAM: " .. spriteName .. " has no facing")
		return
	end
	
	if facing == "S" then
		frontSquare = thisObject:getSquare():getS()
	elseif facing == "E" then
		frontSquare = thisObject:getSquare():getE()
	end
	
	if not frontSquare then
		return false
	end
	
	if not controllerSquare then
		controllerSquare = thisObject:getSquare()
	end

	-- If the front of machine square is valid, walk to it
	if AdjacentFreeTileFinder.privTrySquare(controllerSquare, frontSquare) then
		-- TODO: AdjacentFreeTileFinder didn't work for me on 1st floor for pinball back
		-- Also didn't work on the 2nd floor if I used getOrCreateGridSquare for frontSquare
		--print("PseudoEdPAM: Valid front square, moving")
		ISTimedActionQueue.add(ISWalkToTimedAction:new(thisPlayer, frontSquare))
		return true
	end
	return false
end

-- Do when player selects option to play the game (from context menu)
PAMPlayGameMenu.onPlayGame = function(worldobjects, player, machine, soundFile, soundEnd, actionType, length, boredomReduction, stressReduction)
	if PAMPlayGameMenu.walkToFront(player, machine) then
		ISTimedActionQueue.add(PAMPlayGame:new(player, machine, soundFile, soundEnd, actionType, length, boredomReduction, stressReduction));
	end
end

Events.OnPreFillWorldObjectContextMenu.Add(PAMPlayGameMenu.doBuildMenu);
